@@ -13,7 +13,10 @@ def has_updates():
     :rtype: bool
     """
     cache = apt.Cache()
-    cache.update()
+    try:
+        cache.update()
+    except apt.cache.FetchFailedException:
+        pass
     cache.open(None)
     cache.upgrade(dist_upgrade=True)
     if cache.get_changes():
